@@ -43,8 +43,6 @@ class Keil5P27Converter(Converter):
         '0': 'normal',
         '1': 'core',
         '2': 'reset pin',
-        '3': 'connect under reset',
-        '8': 'core and peripheral',
     }
 
     def __init__(self):
@@ -180,8 +178,8 @@ class Keil5P27Converter(Converter):
             print('Target has no \'Debug Tool\' Option, default to JLink')
             debug_tool_sel = '4'
 
+        # JLink ---------------------------------------------------------------
         if debug_tool_sel == '4':
-            # JLink
             prj_desc['debug']['tool'] = 'jlink'
 
             # 获取 JLink 配置
@@ -197,17 +195,20 @@ class Keil5P27Converter(Converter):
             if jlink_opt is None:
                 print('debug tool not supported, relevant settings will remain default')
             else:
-                # JLink 接口，连接选项
+                # 连接选项 --------------------------------------------------------
                 jlink_opt_itf = re.search('-O([0-9]*)', jlink_opt).group(1)
-                if int(jlink_opt_itf) & (0b1 << 6) == 0:
+                jlink_opt_itf = int(jlink_opt_itf) - 6
+                # 接口类型
+                if jlink_opt_itf & (0b1 << 6) == 0:
                     prj_desc['debug']['interface'] = 'JTAG'
                 else:
                     prj_desc['debug']['interface'] = 'SWD'
-                if int(jlink_opt_itf) & (0b11 << 10) == (0b00 << 10):
+                # 连接
+                if jlink_opt_itf & (0b11 << 10) == (0b00 << 10):
                     prj_desc['debug']['connect'] = 'normal'
-                elif int(jlink_opt_itf) & (0b11 << 10) == (0b01 << 10):
+                elif jlink_opt_itf & (0b11 << 10) == (0b01 << 10):
                     prj_desc['debug']['connect'] = 'with pre-reset'
-                elif int(jlink_opt_itf) & (0b11 << 10) == (0b10 << 10):
+                elif jlink_opt_itf & (0b11 << 10) == (0b10 << 10):
                     prj_desc['debug']['connect'] = 'under reset'
                 else:
                     print('JLink connect type not supported, relevant settings will remain default')
@@ -223,6 +224,22 @@ class Keil5P27Converter(Converter):
                 else:
                     print('JLink reset type not supported, relevant settings will remain default')
 
+                # 烧写选项
+                jlink_dl_opt = re.search('-FO([0-9]*)', jlink_opt).group(1)
+                dl_erase_index = {
+                    17: 'full chip',
+                    1: 'sectors',
+                    0: 'none',
+                }
+                if int(jlink_dl_opt) in dl_erase_index:
+                    prj_desc['debug']['download']['erase'] = dl_erase_index[int(jlink_dl_opt)]
+                else:
+                    print("JLink erase type not supported, default to \'erase sectors\'")
+                    prj_desc['debug']['download']['erase'] = 'sectors'
+                prj_desc['debug']['download']['program'] = bool(int(jlink_dl_opt) & (0b01 << 1) == (0b01 << 1))
+                prj_desc['debug']['download']['verify'] = bool(int(jlink_dl_opt) & (0b01 << 2) == (0b01 << 2))
+                prj_desc['debug']['download']['reset-run'] = bool(int(jlink_dl_opt) & (0b01 << 3) == (0b01 << 3))
+
                 # 工程高级选项 —— flasher configuration
                 jlink_opt_fls = re.findall(
                     '-FF[0-9]([0-9a-zA-Z_]*)(.FLM)? -FS[0-9]([0-9]*) -FL[0-9]([0-9]*)( -FP[0-9]\(\$\$Device:[\S]+\$([\S]+)\))?',
@@ -238,6 +255,106 @@ class Keil5P27Converter(Converter):
                                 prj_desc['advanced_options']['keil_flasher_cfg'] = list()
                             prj_desc['advanced_options']['keil_flasher_cfg'].append(fls_algo)
 
+        # CMSIS-DAP ---------------------------------------------------------------
+        elif debug_tool_sel == '3':
+            prj_desc['debug']['tool'] = 'cmsis-dap'
+
+            # 获取 CMSIS-DAP 配置
+            dap_opt = None
+            try:
+                for item in target_obj_opt['TargetOption']['TargetDriverDllRegistry']['SetRegEntry']:
+                    if item['Key'] == 'CMSIS_AGDI':
+                        dap_opt = item['Name']
+                        break
+            except KeyError:
+                pass
+
+            if dap_opt is None:
+                print('debug tool not supported, relevant settings will remain default')
+            else:
+                # 连接选项 --------------------------------------------------------
+                dap_opt_itf = re.search('-O([0-9]*)', dap_opt).group(1)
+                dap_opt_itf = int(dap_opt_itf) - 398
+                # 接口类型
+                if dap_opt_itf & (0b1 << 6) == 0:
+                    prj_desc['debug']['interface'] = 'JTAG'
+                else:
+                    prj_desc['debug']['interface'] = 'SWD'
+                # 连接
+                if dap_opt_itf & (0b11 << 10) == (0b00 << 10):
+                    prj_desc['debug']['connect'] = 'normal'
+                elif dap_opt_itf & (0b11 << 10) == (0b01 << 10):
+                    prj_desc['debug']['connect'] = 'with pre-reset'
+                elif dap_opt_itf & (0b11 << 10) == (0b10 << 10):
+                    prj_desc['debug']['connect'] = 'under reset'
+                else:
+                    print('JLink connect type not supported, relevant settings will remain default')
+                # 复位
+                if dap_opt_itf & (0b11 << 8) == (0b00 << 8):
+                    prj_desc['debug']['reset'] = 'reset pin'
+                elif dap_opt_itf & (0b11 << 8) == (0b01 << 8):
+                    prj_desc['debug']['reset'] = 'normal'
+                elif dap_opt_itf & (0b11 << 8) == (0b10 << 8):
+                    prj_desc['debug']['reset'] = 'core'
+                else:
+                    print('JLink reset type not supported, relevant settings will remain default')
+
+                # JLink 接口速率
+                dap_opt_spd = re.search('-S([0-9]*)', dap_opt).group(1)
+                dap_speed_index = {
+                    8: 10000,
+                    9: 5000,
+                    10: 2000,
+                    0: 1000,
+                    1: 500,
+                    2: 200,
+                    3: 100,
+                    4: 50,
+                    5: 20,
+                    6: 10,
+                    7: 5,
+                }
+                if int(dap_opt_spd) in dap_speed_index:
+                    prj_desc['debug']['speed'] = dap_speed_index[int(dap_opt_spd)]
+                else:
+                    print('CMSIS-DAP speed not supported, default to 10M')
+                    prj_desc['debug']['speed'] = 8
+
+                # 烧写选项
+                dap_dl_opt = re.search('-FO([0-9]*)', dap_opt).group(1)
+                dl_erase_index = {
+                    17: 'full chip',
+                    1: 'sectors',
+                    0: 'none',
+                }
+                if int(dap_dl_opt) in dl_erase_index:
+                    prj_desc['debug']['download']['erase'] = dl_erase_index[int(dap_dl_opt)]
+                else:
+                    print("CMSIS-DAP erase type not supported, default to \'erase sectors\'")
+                    prj_desc['debug']['download']['erase'] = 'sectors'
+                prj_desc['debug']['download']['program'] = bool(int(dap_dl_opt) & (0b1 << 1) == (0b01 << 1))
+                prj_desc['debug']['download']['verify'] = bool(int(dap_dl_opt) & (0b1 << 2) == (0b01 << 2))
+                prj_desc['debug']['download']['reset-run'] = bool(int(dap_dl_opt) & (0b1 << 3) == (0b01 << 3))
+
+                # 工程高级选项 —— flasher configuration
+                dap_opt_fls = re.findall(
+                    '-FF[0-9]([0-9a-zA-Z_]*)(.FLM)? -FS[0-9]([0-9]*) -FL[0-9]([0-9]*)( -FP[0-9]\(\$\$Device:[\S]+\$([\S]+)\))?',
+                    dap_opt)
+
+                if dap_opt_fls is not None:
+                    for fls_name, fls_ext, fls_base, fls_size, _, fls_path in dap_opt_fls:
+                        fls_algo = dict(name=fls_name + fls_ext, path=fls_path if fls_path != '' else None,
+                                        base=fls_base,
+                                        size=fls_size)
+                        if fls_algo not in target_chip['target']['keil_5_flash_algorithms']:
+                            if prj_desc['advanced_options']['keil_flasher_cfg'] is None:
+                                prj_desc['advanced_options']['keil_flasher_cfg'] = list()
+                            prj_desc['advanced_options']['keil_flasher_cfg'].append(fls_algo)
+
+        # 不支持的仿真器 ---------------------------------------------------------------
+        else:
+            print('Debugger not supported, default to JLink')
+
         # 编译器全局定义
         try:
             defines = target_obj['TargetOption']['TargetArmAds']['Cads']['VariousControls']['Define']
@@ -259,6 +376,27 @@ class Keil5P27Converter(Converter):
             print('Target has no \'Includes\', default to an empty list')
             prj_desc['includePaths'] = []
 
+        # 编译器全局定义(ASM)
+        try:
+            defines = target_obj['TargetOption']['TargetArmAds']['Aads']['VariousControls']['Define']
+            define_list = str.split(defines, ",")
+            for define in define_list:
+                if define in target_chip['defines']:
+                    define_list.remove(define)
+            prj_desc['definesASM'] = define_list
+        except KeyError:
+            print('Target has no \'Defines(ASM)\', default to an empty list')
+            prj_desc['definesASM'] = []
+
+        # 包含路径(ASM)
+        try:
+            includes = target_obj['TargetOption']['TargetArmAds']['Aads']['VariousControls']['IncludePath']
+            include_list = str.split(includes, ';')
+            prj_desc['includePathsASM'] = include_list
+        except KeyError:
+            print('Target has no \'Includes(ASM)\', default to an empty list')
+            prj_desc['includePathsASM'] = []
+
         # 文件组
         prj_desc['groups'] = list()
         try:
