@@ -25,6 +25,8 @@ class Keil5Generator(Generator):
 
     @staticmethod
     def combine_files(base_files: list, append_files: list):
+        if append_files is None:
+            return
         if len(append_files) != 0:
             for append_file in append_files:
                 find_flag = False
@@ -45,7 +47,10 @@ class Keil5Generator(Generator):
                 for base_group in base_groups:
                     # 有相同组，合并这两组的文件，并递归合并子组
                     if base_group['name'] == append_group['name']:
-                        self.combine_files(base_group['files'], append_group['files'])
+                        if 'files' in append_group and append_group['files'] is not None:
+                            if 'files' not in base_group:
+                                base_group['files'] = []
+                            self.combine_files(base_group['files'], append_group['files'])
                         if 'groups' in append_group:
                             if 'groups' not in base_group:
                                 base_group['groups'] = []
@@ -95,8 +100,10 @@ class Keil5Generator(Generator):
                     cvt_desc = cvt.analyze_project({'uvprojx': f_prj_name, 'uvoptx': f_opt_name})
                     # 合并预处理宏定义
                     prj_info['defines'] = list(set(prj_info['defines'] + cvt_desc['defines']))
+                    prj_info['definesASM'] = list(set(prj_info['definesASM'] + cvt_desc['definesASM']))
                     # 合并包含路径
                     prj_info['includePaths'] = list(set(prj_info['includePaths'] + cvt_desc['includePaths']))
+                    prj_info['includePathsASM'] = list(set(prj_info['includePathsASM'] + cvt_desc['includePathsASM']))
                     # 合并工程组
                     self.combine_groups(prj_info['groups'], cvt_desc['groups'])
 
