@@ -48,12 +48,19 @@ class Converter:
 
     # 分析工程抽象接口
     def analyze_project(self, prj_files: dict, target_name: str = '', output_file: str = '') -> dict:
+        """
+        分析目标工程并返回工程描述信息。
+
+        :param prj_files: 工程文件路径字典，所需内容依据不同的解析器的实现有所不同。
+        :param target_name: （可选，部分解析器指定此项无效）目标名称。如果一个工具链工程中有多个目标，可以通过该项指定需要转换哪个目标。默认转换首个搜索到的目标。
+        :param output_file: （可选）输出文件路径。指定该项可以将工程描述信息到处到文件中（yaml格式）。
+        """
         pass
 
 
 class Generator:
     # 生成工程抽象接口
-    def generate(self, prj_info, target_info, prj_path, version):
+    def generate(self, prj_info, target_info, prj_path, version, **kwargs):
         pass
 
     # 获取生成工程的名称
@@ -92,7 +99,7 @@ class ProjectGenerator:
         self.__prj_desc = converter.analyze_project(req, output_file=output_file)
         return self.__prj_desc
 
-    def generate(self, dest_prj_path, generator: Generator, generator_version, input_desc=None):
+    def generate(self, dest_prj_path, generator: Generator, generator_version, input_desc=None, **kwargs):
         # 加载工程描述
         if input_desc is None:
             # 使用已有内容进行转换
@@ -134,4 +141,4 @@ class ProjectGenerator:
         self.__prj_desc['defines'] = chip['defines'] + self.__prj_desc['defines']
 
         # 生成工程
-        generator.generate(self.__prj_desc, chip['target'], dest_prj_path, generator_version)
+        generator.generate(self.__prj_desc, chip['target'], dest_prj_path, generator_version, **kwargs)
